from ..ui import *
from .link import Window as Link


class Panel(Component):
    def __init__(self, parent):
        super().__init__(parent, 0, 0)

        self.peers = { }

    def add_peer(self, label):
        if label in self.peers:
            self.peers[label][0] = True

        else:
            self.peers[label] = [True, None]

        self.refresh_peers()

    def del_peer(self, label):
        if label not in self.peers:
            return

        self.peers[label][0] = False
        self.refresh_peers()

    def refresh_peers(self):
        clock.schedule_once(self._refresh_peers_, 0)

    def _refresh_peers_(self, dt):
        y = 30
        for name, data in self.peers.items():
            keep = data[0]
            widgets = data[1]

            if widgets:
                widgets[0].delete()
                widgets[1].remove()
                #widgets[2].remove()
                data[1] = None

            if keep:
                def link_push(peers, name):
                    def pusher():
                        peers[name][1][2].set_visible(True)

                    return pusher

                btn_link = Button(self, 10, y,
                    image('btn_link.png'),
                    image('btn_link-push.png'),
                    image('btn_link-hover.png'))
                btn_link.on_press = link_push(self.peers, name)

                data[1] = [
                    Label(self, 40, y+8, name),
                    btn_link,
                    Link(self.win.app, name)
                ]
                y += 30




from ..ui import *

import lank.name


class Window(LWin):
    def __init__(self, app, label):
        super().__init__(app, f'Link to {label} | lanku', 700, 300, 'link')
        self.label = label

        self.items = [ ]

    def on_show(self):
        super().on_show()
        self.app.node.get_history(self.label, self.refresh)

    def refresh(self, history):
        clock.schedule_once(self._refresh_, 0, history)

    def _refresh_(self, dt, history):
        for item in self.items:
            if item[0]: item[0].remove()
            item[1].delete()

        self.items = [ ]
        y = 3

        for item in history.items:
            dt = item._to_datetime_(item.timestamp).astimezone().strftime(
                '%a, %d %b %Y @ %I:%M %p')

            if item.name == lank.name.REGISTER:
                self.items.append((None, Label(self, 130, y+5,
                    f'{dt}: Created new key-pair')))

            elif item.name == lank.name.PEER:
                msg = f'{dt}: Signed on'
                if ':' in item.key[item.key.index(':')+1:]:
                    alias = item.key[item.key.index(':')+1:]
                    alias = alias[alias.index(':')+1:]
                    msg += f' as {alias}'
                msg += f' at {item.address}'
                self.items.append((
                    Button(self, 3, y,
                        image('btn_connect.png'),
                        image('btn_connect-push.png'),
                        image('btn_connect-hover.png')),
                    Label(self, 130, y+5, msg)))

            else:
                self.items.append((None, Label(self, 3, y+5,
                    f'{dt}: Did something')))

            y += 30

        clock.schedule_once(self.null, 0.1)

    def null(self, dt):
        self.place()




from .__version__ import __version__
from .home import Window as Home
from .node import Client as Node
from .peer import Master as Peer

import lank.name
import pyglet.app
from PIL import Image, ImageDraw

from threading import Thread
import sys


class Application:
    def __init__(self):
        print(f'lanku v{__version__}')
        self.icon = create_image(64, 64, 'green', 'white')
        try:
            from .systray import Icon as TrayIcon
            self.tray_icon = TrayIcon(self)
            self.tray_thread = Thread(target=self.tray_icon.run)

        except Exception as e:
            em = str(e)
            if not em: em = type(e)
            print(f'WARNING - Unable to setup system tray icon: {em}')
            self.tray_icon = None
            self.tray_thread = None

        self.node = None

        self.home = Home(self)

        self.finished = False
        self.exiting = False

    def run(self):
        try:
            if self.tray_thread:
                self.tray_thread.start()
            self.show_hide()
            pyglet.app.run()
            self.finished = True

        except KeyboardInterrupt:
            pass

        finally:
            self.quit()
            if self.tray_thread:
                self.tray_thread.join()

    def show_hide(self):
        if self.home.minimized:
            self.home.set_visible(True)

        else:
            self.home.set_visible(not self.home.visible)

    def quit(self):
        if self.exiting: return
        self.exiting = True
        print('quit')

        if self.tray_icon:
            self.tray_icon.stop()

        if not self.finished:
            self.home.save()

        self.disconnect()
        pyglet.app.exit()

    def connect(self):
        print('connecting')
        #print(f'pass: {self.home.pnl_config.pwd.value}')

        self.node = Node(self)
        self.node.start()

    def disconnect(self):
        if not self.node: return
        print('disconnecting')
        node = self.node
        self.node = None
        node.stop()
        node.join()
        print('finished')

    def notify(self, signed):
        if signed.name == lank.name.REGISTER:
            self._notify_(signed.label, f'{signed.label} has changed keys.')

        elif signed.name == lank.name.PEER:
            msg = f'{signed.label} signed on'
            if ':' in signed.key[signed.key.index(':')+1:]:
                alias = signed.key[signed.key.index(':')+1:]
                alias = alias[alias.index(':')+1:]
                msg += f' as {alias}'
            msg += '.'
            self._notify_(signed.label, msg)

        else:
            self._notify_(signed.label, f'{signed.label} has done something.')

    def _notify_(self, title, msg):
        if self.tray_icon:
            self.tray_icon.notify(msg, f'{title} | lanku')

        else:
            print(f'@@ NOTICE [{title}] @@')
            print(f'   >>   {msg}')


def create_image(width, height, color1, color2):
    image = Image.new('RGBA', (width, height), color1)
    dc = ImageDraw.Draw(image)
    dc.rectangle(
        (width // 2, 0, width, height // 2),
        fill=color2)
    dc.rectangle(
        (0, height // 2, width // 2, height),
        fill=color2)

    return image

